/*
Author: Matthieu Herwegh
Date: 20/07/2016
Subject: Communication I2C
*/
#ifndef F_CPU 
#define F_CPU 16000000UL
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <math.h>
#include <stdlib.h>

#include "i2c_master.h"

#define 		TEMP_ADDRESS	0x80
#define 		TEMP_REG		0xE3
#define			TEMP_LENGTH		2
#define 		PRESS_ADDRESS	0x78	
#define			PRESS_REG		0x00	//Honeywell doesn't implement register address, we just need to send the adress of the device.
#define			PRESS_LENGTH	2
#define			HUMI_ADDRESS	0x80
#define			HUMI_REG		0xE5
#define			HUMI_LENGTH		2

#define			ERROR_LIMIT		5		//The limit of count of timeout/error from a sensor before we send an error code to the user.
#define 		BAUD 			9600	//Baudrate of the serial line (to send the data from the sensors over the serial line)
#define 		DELAY_INIT 		50		//delay between steps of initialization (waiting that the devices power up)



void init_uart(uint16_t baudrate){

	uint16_t UBRR_val = (F_CPU/16)/(baudrate-1);

	UBRR0H = UBRR_val >> 8;
	UBRR0L = UBRR_val;

	UCSR0B |= (1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0); // UART TX (Transmit - senden) einschalten
	UCSR0C |= (1<<USBS0) | (3<<UCSZ00); //Modus Asynchron 8N1 (8 Datenbits, No Parity, 1 Stopbit)
}

void uart_putc(unsigned char c){

	while(!(UCSR0A & (1<<UDRE0))); // wait until sending is possible
	UDR0 = c; // output character saved in c
}

void uart_puts(char *s){
	while(*s){
		uart_putc(*s);
		s++;
	}
}

void init_tempSensor()
{
// soft reset takes less tahn 15ms 
	char init[1] = 0xFE;
	i2c_transmit(TEMP_ADDRESS, init, sizeof(init));
	_delay_ms(DELAY_INIT);

//set up the sensor
	i2c_readReg(TEMP_ADDRESS, 0xE7, init, sizeof(init));
	init[0] = init[0] | 0x81;			//0x81 give a response time below 15ms with a correct accuracy
	i2c_writeReg(TEMP_ADDRESS, 0xE6, init, sizeof(init));
}


void main()
{
	init_uart(BAUD);
	i2c_init();
	_delay_ms(DELAY_INIT);

	//initialize the temperature sensor
	init_tempSensor();

	short error_temp, error_press, error_humi;

	char temp[TEMP_LENGTH];
	char press[PRESS_LENGTH];
	char humi[HUMI_LENGTH];

	while (1)
	{
	//read the values
		if (i2c_readReg(TEMP_ADDRESS, TEMP_REG, temp, sizeof(temp)))
			error_temp++;
		else
			error_temp = 0;

		if (i2c_receive(PRESS_ADDRESS, press, sizeof(press)))// pressure sensor from HoneyWell doesn't use the system of register
			error_press++;
		else
			error_press = 0;

		if (i2c_readReg(HUMI_ADDRESS, HUMI_REG, press, sizeof(humi)))
			error_humi++;
		else
			error_humi = 0;


	// If ERROR_LIMIT errors in a row, send an error code for this sensor
		if (error_temp > ERROR_LIMIT)
		{
			uart_putc(TEMP_ADDRESS);
			uart_putc(0x55);
			uart_putc(0xff);
		}
		else
		{
			uart_putc(TEMP_ADDRESS);
			uart_putc(TEMP_REG);
			uart_puts(temp);
		}
		_delay_ms(100);

	// If ERROR_LIMIT errors in a row, send an error code for this sensor
		if (error_press > ERROR_LIMIT)
		{
			uart_putc(press_ADDRESS);
			uart_putc(0x55);
			uart_putc(0xff);
		}
		else
		{
			uart_putc(PRESS_ADDRESS);
			uart_putc(PRESS_REG);
			uart_puts(press);
		}
		_delay_ms(100);
		
	// If ERROR_LIMIT errors in a row, send an error code for this sensor
		if (error_humi > ERROR_LIMIT)
		{
			uart_putc(HUMI_ADDRESS);
			uart_putc(0x55);
			uart_putc(0xff);
		}
		else
		{
			uart_putc(HUMI_ADDRESS);
			uart_putc(HUMI_REG);
			uart_puts(humi);
		}
		_delay_ms(100);

	}
}
